#include <math.h>
#include <stdlib.h>
#include "nrutil.c"

#define N 4


float **y, *x; 

float K, L, G;


void euler_step (float x, 
	 			 float y[], 
				 float h, 
				 void (*f) (float, float [], float []), 
				 float ynext[])
{
	float *dyperdx;
	int n;

	dyperdx = vector (1, N);

	(*f) (x, y, dyperdx);

	for (n=1; n<=N; n++)	
	{
		ynext[n] = y[n] + h*dyperdx[n];  
	}

	free_vector (dyperdx, 1, N);
}


void euler_drv (float x0, 
				float x1, 
				int xstep,
				float y0[],
				void (*f) (float, float [], float []))
{
	float h, *yn, *yn1;
	int i, n;

	h = (x1-x0)/xstep;

	yn = vector (1, N);
	yn1 = vector (1, N);

	x[1] = x0;

	for (n=1; n<=N; n++) 
	{	
		y[n][1] = y0[n];
		yn[n] = y0[n];
	}

	for (i=1; i<=xstep; i++)
	{
		euler_step (x[i], 
					yn, 
					h, 
					f, 
					yn1);

		x[i+1] = x[i]+h;
		
		for (n=1; n<=N; n++) 
		{	
			y[n][i+1] = yn1[n];
			yn[n] = yn1[n];
		}
	}

	free_vector (yn, 1, N);
	free_vector (yn1, 1, N);

}


void f_rugos_inga (float x, float y[], float dyperdx[])
{
	float s;

	s = sqrt( y[1]*y[1] + y[2]*y[2] );
	
	dyperdx[1] = y[3];
	dyperdx[2] = y[4];
	if (s>0)
	{	
		dyperdx[3] = (K*(L-s)*y[1])/s;
		dyperdx[4] = (K*(L-s)*y[2])/s - G;
	}
	else nrerror("Zeroval osztas: f_rugos_inga !");
}


void main (int argc, char *argv[])
{

	int i, xstep;
	float x0, x1, *y0, nrg;


	G = 10.0;					// a fizikai problema alapertelmezett parameterei
	K = 10.0;
	L = 1.0;

	x0 = 0.0;
	x1 = 1.0;
	xstep = 1024;

	y0 = vector (1, N);			// kezdeti ertekek es alapertelmezett ertekei
	y0[1] = L;
	y0[2] = 0;
	y0[3] = 0;
	y0[4] = 0;

	for (i=1; i<argc; i++ ){
		
		switch(i){
	    case 1: 
			x0 = atof(argv[1]); 
			break;
		case 2:
			x1 = atof(argv[2]); 
			break;
		case 3:
			xstep = atoi(argv[3]); 
			break;
		case 4:
			y0[1] = atof(argv[4]);
			break;
		case 5:
			y0[2] = atof(argv[5]);
			break;
		case 6:
			y0[3] = atof(argv[6]);
			break;
		case 7:
			y0[4] = atof(argv[7]);
			break;
		case 8:
			K = atof(argv[8]);
			break;
		case 9:
			L = atof(argv[9]);
			break;
		} 
	}

	x = vector (1, xstep+1);
	y = matrix (1, N, 1, xstep+1);


	euler_drv (	x0, 
				x1, 
				xstep,
				y0,
				f_rugos_inga);

	for (i=1;i<=xstep+1;i++)
	{
		nrg = (sqrt(y[1][i]*y[1][i] + y[2][i]*y[2][i]) - L);
		nrg = (y[3][i]*y[3][i] + y[4][i]*y[4][i])/2 
			+ G*y[2][i] 
			+ K*(nrg*nrg)/2;
		printf("%f\t%f\t%f\t%f\t%f\t%f\n", x[i], y[1][i], y[2][i], y[3][i], y[4][i], nrg);
	}

	free_vector (x, 1, xstep+1);
	free_vector (y0, 1, xstep+1);
	free_matrix (y, 1, N, 1, xstep+1);
}
